@@ -80,10 +80,10 @@ static char *mksquashfs_options[]={
 	"-default-mode", "-default-uid", "-default-gid", "-ignore-zeros", "",
 	"", "",
 	/* expert options */
-	"-nopad", "-offset", "-o", "", "", "",
+	"-stream", "-fix", "-nopad", "-offset", "-o", "", "", "",
 	/* help options */
 	"-help", "-help-option", "-help-section", "-help-comp", "-help-all",
-	"-Xhelp", "-h", "-ho", "-hs", "-ha", "", "", "",
+	"-Xhelp", "-h", "-ho", "-hs", "-ha", "-no-pager", "-cols", "", "", "",
 	/* misc options */
 	"-fstime", "-always-use-fragments", "-root-owned",
 	"-noInodeCompression", "-noIdTableCompression", "-noDataCompression",
@@ -116,10 +116,10 @@ static char *sqfstar_options[]={
 	"-processors", "-mem", "-mem-percent", "-mem-default", "-overcommit",
 	"", "", "",
 	/* expert options */
-	"-nopad", "-offset", "-o", "", "", "",
+	"-stream", "-fix", "-nopad", "-offset", "-o", "", "", "",
 	 /* help options */
 	 "-help", "help-option", "-help-section", "-help-comp", "-help-all",
-	 "-Xhelp", "-h", "-ho", "-hs", "-ha", "", "", "",
+	 "-Xhelp", "-h", "-ho", "-hs", "-ha", "-no-pager", "-cols", "", "", "",
 	/* misc options */
 	"-fstime", "-root-owned", "-noInodeCompression",
 	"-noIdTableCompression", "-noDataCompression", "-noFragmentCompression",
@@ -156,10 +156,10 @@ static char *mksquashfs_args[]={
 	/* tar options */
 	"<mode>", "<value>", "<value>", "", "", "", "",
 	/* expert options */
-	"", "<offset>", "<offset>", "", "", "",
+	"", "<filesystem>", "", "<offset>", "<offset>", "", "", "",
 	/* help options */
 	"", "<regex>", "<section>", "<comp>", "", "", "", "<regex>",
-	"<section>", "", "", "", "",
+	"<section>", "", "", "<width>", "", "", "",
 	/* misc options */
 	"<time>", "", "", "", "", "", "", "", ""
 };
@@ -184,10 +184,10 @@ static char *sqfstar_args[]={
 	"<percentage>", "<number>", "<size>", "<percent>", "", "<percentage>",
 	"", "", "",
 	/* expert options */
-	"", "<offset>", "<offset>", "", "", "",
+	"", "<filesystem>", "", "<offset>", "<offset>", "", "", "",
 	/* help options */
 	"", "<regex>", "<section>", "<comp>", "", "", "", "<regex>",
-	"<section>", "" "", "", "",
+	"<section>", "", "", "<width>", "", "", "",
 	/* misc options */
 	"<time>", "", "", "", "", "", "", ""
 };
@@ -446,13 +446,26 @@ static char *mksquashfs_text[]={
 		"blocks\n",
 	"\n", "Expert options (these may make the filesystem unmountable):",
 	"\n",
+	"-stream\t\t\toutput the filesystem to STDOUT rather than to a file."
+		"  This allows the output to be piped to another program or "
+		"elsewhere with ssh.  The resultant streamed Squashfs "
+		"filesystem will not be a normal Squashfs filesystem because "
+		"the super block will be at the end of the filesystem.  But "
+		"it can be fixed up afterwards with the -fix option and "
+		"Unsquashfs 4.7.3 can read streamed unfixed up filesystems "
+		"directly.  Use - for the output FILESYSTEM on the command "
+		"line.  Duplicate detection is disabled using the -stream "
+		"option\n",
+	"-fix <filesystem>\tfix <filesystem> generated using the -stream "
+		"option\n",
 	"-nopad\t\t\tdo not pad filesystem to a multiple of 4K\n",
 	"-offset <offset>\tskip <offset> bytes at the beginning of FILESYSTEM."
 		"  Optionally a suffix of K, M or G can be given to specify "
 		"Kbytes, Mbytes or Gbytes respectively.  Default 0 bytes\n",
 	"-o <offset>\t\tsynonym for -offset\n",
 	"\n", "Help options:", "\n",
-	"-help\t\t\tprint help summary information to stdout\n",
+	"-help\t\t\tprint help summary information to pager (or stdout if not "
+		"a terminal)\n",
 	"-help-option <regex>\tprint the help information for options matching "
 		"<regex> to pager (or stdout if not a terminal)\n",
 	"-help-section <section>\tprint the help information for section "
@@ -472,6 +485,9 @@ static char *mksquashfs_text[]={
 	"-ho <regex>\t\tshorthand alternative to -help-option\n",
 	"-hs <section>\t\tshorthand alternative to -help-section\n",
 	"-ha\t\t\tshorthand alternative to -help-all\n",
+	"-no-pager\t\tdo not use a pager to output help information\n",
+	"-cols <width>\t\tuse <width> columns to output help information.  "
+		"Useful if output is not to a terminal\n",
 	"\n", "Miscellaneous options:", "\n",
 	"-fstime <time>\t\talternative name for -mkfs-time\n",
 	"-always-use-fragments\talternative name for -tailends\n",
@@ -553,12 +569,18 @@ static char *mksquashfs_text[]={
 	"  1\tFatal errors occurred, Mksquashfs aborted and did not generate a "
 		"filesystem (or update if appending).\n",
 	"\n","See also (extra information elsewhere):", "\n",
-	"The README for the Squashfs-tools 4.7 release, describing the new "
-		"features can be read here https://github.com/plougher/"
-		"squashfs-tools/blob/master/Documentation/4.7/README\n",
-	"\nThe Squashfs-tools USAGE guides and other documentation can be read "
-		"here https://github.com/plougher/squashfs-tools/blob/master/"
-		"Documentation/4.7\n",
+	"The README for the Squashfs-tools 4.7.3 release, describing the "
+		"new features can be read here https://github.com/plougher/"
+		"squashfs-tools/blob/master/Documentation/4.7.3/README.md\n",
+	"\nThe MKSQUASHFS usage guide can be read here https://github.com/"
+		"plougher/squashfs-tools/blob/master/Documentation/4.7.3/"
+		"USAGE-MKSQUASHFS.md\n",
+	"\nThe CHANGELOG for Squashfs tools can be read here "
+		"https://github.com/plougher/squashfs-tools/blob/master/"
+		"CHANGES.md\n",
+	"\nThe ACTIONS-README for the actions subsystem can be read here "
+		"https://github.com/plougher/squashfs-tools/blob/master/"
+		"Documentation/4.7.3/ACTIONS-README.md\n",
 	NULL
 };
 
@@ -740,6 +762,17 @@ static char *sqfstar_text[]={
 		"granularity of 25%, and 20 processors have a percentage "
 		"granularity of 5%\n",
 	"\n", "Expert options (these may make the filesystem unmountable):", "\n",
+	"-stream\t\t\toutput the filesystem to STDOUT rather than to a file."
+		"  This allows the output to be piped to another program or "
+		"elsewhere with ssh.  The resultant streamed Squashfs "
+		"filesystem will not be a normal Squashfs filesystem because "
+		"the super block will be at the end of the filesystem.  But "
+		"it can be fixed up afterwards with the -fix option and "
+		"Unsquashfs 4.7.3 can read streamed unfixed up filesystems "
+		"directly.  Duplicate detection is disabled using the -stream "
+		"option\n",
+	"-fix <filesystem>\tfix <filesystem> generated using the -stream "
+		"option\n",
 	"-nopad\t\t\tdo not pad filesystem to a multiple of 4K\n",
 	"-offset <offset>\tskip <offset> bytes at the beginning of "
 		"FILESYSTEM.  Optionally a suffix of K, M or G can be given to "
@@ -747,7 +780,8 @@ static char *sqfstar_text[]={
 		"bytes\n",
 	"-o <offset>\t\tsynonym for -offset\n",
 	"\n", "Help options:", "\n",
-	"-help\t\t\tprint help summary information to stdout\n",
+	"-help\t\t\tprint help summary information to pager (or stdout if not "
+		"a terminal)\n",
 	"-help-option <regex>\tprint the help information for options matching "
 		"<regex> to pager (or stdout if not a terminal)\n",
 	"-help-section <section>\tprint the help information for section "
@@ -767,6 +801,9 @@ static char *sqfstar_text[]={
 	"-ho <regex>\t\tshorthand alternative to -help-option\n",
 	"-hs <section>\t\tshorthand alternative to -help-section\n",
 	"-ha\t\t\tshorthand alternative to -help-all\n",
+	"-no-pager\t\tdo not use a pager to output help information\n",
+	"-cols <width>\t\tuse <width> columns to output help information.  "
+		"Useful if output is not to a terminal\n",
 	"\n", "Miscellaneous options:", "\n",
 	"-fstime <time>\t\talternative name for mkfs-time\n",
 	"-root-owned\t\talternative name for -all-root\n",
@@ -846,12 +883,15 @@ static char *sqfstar_text[]={
 	"  1\tFatal errors occurred, Sqfstar aborted and did not generate a "
 		"filesystem.\n",
 	"\n","See also (extra information elsewhere):", "\n",
-	"The README for the Squashfs-tools 4.7 release, describing the new "
-		"features can be read here https://github.com/plougher/"
-		"squashfs-tools/blob/master/Documentation/4.7/README\n",
-	"\nThe Squashfs-tools USAGE guides and other documentation can be read "
-		"here https://github.com/plougher/squashfs-tools/blob/master/"
-		"Documentation/4.7\n",
+	"The README for the Squashfs-tools 4.7.3 release, describing the "
+		"new features can be read here https://github.com/plougher/"
+		"squashfs-tools/blob/master/Documentation/4.7.3/README.md\n",
+	"\nThe Sqfstar usage guide can be read here https://github.com/"
+		"plougher/squashfs-tools/blob/master/Documentation/4.7.3/"
+		"USAGE-SQFSTAR.md\n",
+	"\nThe CHANGELOG for Squashfs tools can be read here "
+		"https://github.com/plougher/squashfs-tools/blob/master/"
+		"CHANGES.md\n",
 	NULL
 };
 
@@ -859,15 +899,7 @@ static void print_help_all(char *name, char *syntax, char **options_text)
 {
 	int i, cols;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	autowrap_printf(pager, cols, syntax, name);
 
@@ -876,11 +908,7 @@ static void print_help_all(char *name, char *syntax, char **options_text)
 
 	display_compressor_usage(pager, COMP_DEFAULT, cols);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(0);
 }
 
@@ -890,7 +918,7 @@ static void print_option(char *prog_name, char *opt_name, char *pattern, char **
 {
 	int i, res, matched = FALSE;
 	regex_t *preg = MALLOC(sizeof(regex_t));
-	int cols = get_column_width();
+	int cols;
 	pid_t pager_pid;
 	FILE *pager;
 
@@ -899,14 +927,12 @@ static void print_option(char *prog_name, char *opt_name, char *pattern, char **
 		char str[1024]; /* overflow safe */
 
 		regerror(res, preg, str, 1024);
+		cols = get_column_width();
 		autowrap_printf(stderr, cols, "%s: %s invalid regex %s because %s\n", prog_name, opt_name, pattern, str);
 		exit(1);
 	}
 
-	if(isatty(STDOUT_FILENO))
-		pager = exec_pager(&pager_pid);
-	else
-		pager = stdout;
+	pager = launch_pager(&pager_pid, &cols);
 
 	for(i = 0; options[i] != NULL; i++) {
 		res = regexec(preg, options[i], (size_t) 0, NULL, 0);
@@ -918,11 +944,7 @@ static void print_option(char *prog_name, char *opt_name, char *pattern, char **
 		}
 	}
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 
 	if(!matched) {
 		autowrap_printf(stderr, cols, "%s: %s %s does not match any %s option\n", prog_name, opt_name, pattern, prog_name);
@@ -958,16 +980,8 @@ static void print_section(char *prog_name, char *opt_name, char *sec_name, char
 {
 	int i, j, secs, cols, res, matched = FALSE;
 	pid_t pager_pid;
-	FILE *pager;
 	regex_t *preg;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	if(strcmp(sec_name, "list") == 0) {
 		autowrap_printf(pager, cols, "\nUse following section name to print %s help information for that section\n\n", prog_name);
@@ -985,10 +999,7 @@ static void print_section(char *prog_name, char *opt_name, char *sec_name, char
 	if(res) {
 		char str[1024]; /* overflow safe */
 
-		if(pager != stdout) {
-			fclose(pager);
-			wait_to_die(pager_pid);
-		}
+		delete_pager(pager, pager_pid);
 
 		regerror(res, preg, str, 1024);
 		autowrap_printf(stderr, cols, "%s: %s invalid regex %s because %s\n", prog_name, opt_name, sec_name, str);
@@ -1025,11 +1036,7 @@ exact_match:
 	}
 
 finish:
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(0);
 }
 
@@ -1038,15 +1045,7 @@ static void handle_invalid_option(char *prog_name, char *opt_name, char **sectio
 {
 	int cols;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	autowrap_printf(pager, cols, "%s: %s is an invalid option\n\n", prog_name, opt_name);
 	autowrap_printf(pager, cols, "Run\n  \"%s -help-option <regex>\" to get help on all options matching <regex>\n", prog_name);
@@ -1054,11 +1053,7 @@ static void handle_invalid_option(char *prog_name, char *opt_name, char **sectio
 	print_section_names(pager, "\t", cols, sections, options_text);
 	autowrap_printf(pager, cols, "\nOr run\n  \"%s -help-all\" to get help on all the sections\n", prog_name);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(1);
 }
 
@@ -1067,15 +1062,7 @@ static void print_help(char *prog_name, char *message, char *syntax, char **sect
 {
 	int cols;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	if(message)
 		autowrap_print(pager, message, cols);
@@ -1085,11 +1072,7 @@ static void print_help(char *prog_name, char *message, char *syntax, char **sect
 	print_section_names(pager, "\t", cols, sections, options_text);
 	autowrap_printf(pager, cols, "\nOr run\n  \"%s -help-all\" to get help on all the sections\n", prog_name);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(message == NULL ? 0 : 1);
 }
 
@@ -1099,15 +1082,7 @@ static void print_option_help(char *prog_name, char *option, char **sections, ch
 	int cols;
 	char *string;
 	pid_t pager_pid;
-	FILE *pager;
-
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	FILE *pager = launch_pager(&pager_pid, &cols);
 
 	VASPRINTF(&string, fmt, ap);
 	autowrap_print(pager, string, cols);
@@ -1118,11 +1093,7 @@ static void print_option_help(char *prog_name, char *option, char **sections, ch
 	autowrap_printf(pager, cols, "\nOr run\n  \"%s -help-all\" to get help on all the sections\n", prog_name);
 	free(string);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(1);
 }
 
@@ -1223,23 +1194,13 @@ void print_compressor_options(char *comp_name, char *prog_name)
 		exit(1);
 	}
 
-	if(isatty(STDOUT_FILENO)) {
-		cols = get_column_width();
-		pager = exec_pager(&pager_pid);
-	} else {
-		cols = 80;
-		pager = stdout;
-	}
+	pager = launch_pager(&pager_pid, &cols);
 
 	if(strcmp(comp_name, "list") == 0)
 		autowrap_print(pager, "\t" COMPRESSORS "\n", cols);
 	else
 		print_comp_options(pager, cols, comp_name, prog_name);
 
-	if(pager != stdout) {
-		fclose(pager);
-		wait_to_die(pager_pid);
-	}
-
+	delete_pager(pager, pager_pid);
 	exit(0);
 }
